<?php

declare(strict_types=1);

namespace Drupal\field_group_nav\Plugin\field_group\FieldGroupFormatter;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\field_group\FieldGroupFormatterBase;
use Drupal\field_group_nav\NavigationBuilderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Nav formatter.
 *
 * @FieldGroupFormatter(
 *   id = "nav_item",
 *   label = @Translation("Nav item"),
 *   description = @Translation("Provide navigation links within the current document"),
 *   supported_contexts = {
 *     "view",
 *     "form"
 *   }
 * )
 */
class NavItem extends FieldGroupFormatterBase implements ContainerFactoryPluginInterface {

  /**
   * Navigation builder.
   *
   * @var \Drupal\field_group_nav\NavigationBuilderInterface
   */
  protected NavigationBuilderInterface $navigationBuilder;

  /**
   * Constructs a Nav item object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param object $group
   *   The group object.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label.
   * @param \Drupal\field_group_nav\NavigationBuilderInterface $navigation_builder
   *   Navigation builder.
   */
  public function __construct($plugin_id, $plugin_definition, $group, array $settings, $label, NavigationBuilderInterface $navigation_builder) {
    parent::__construct($plugin_id, $plugin_definition, $group, $settings, $label);
    $this->navigationBuilder = $navigation_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['group'],
      $configuration['settings'],
      $configuration['label'],
      $container->get('field_group_nav.navigation_builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm() {
    $form = parent::settingsForm();
    $class = self::class;

    $form['nav_classes'] = [
      '#title' => $this->t('CSS classes for navigation item'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('nav_classes'),
      '#weight' => 11,
      '#element_validate' => [[$class, 'validateCssClass']],
    ];

    // Element id is generated by plugin.
    unset($form['id']);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    if ($this->getSetting('nav_classes')) {
      $summary[] = $this->t('CSS classes for navigation item: @classes', ['@classes' => $this->getSetting('nav_classes')]);
    }

    $summary[] = $this->t('Group name: @name', ['@name' => $this->group->group_name]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function process(&$element, $processed_object) {
    parent::preRender($element, $processed_object);

    $element += [
      '#type' => 'field_group_nav_item',
      '#title' => $this->getLabel(),
      '#id' => $this->navigationBuilder->getId($element['#group_name']),
      '#multilingual' => TRUE,
    ];

    $classes = $this->getClasses();
    if (!empty($classes)) {
      $element += ['#attributes' => ['class' => $classes]];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preRender(&$element, $rendering_object) {
    parent::preRender($element, $rendering_object);
    $this->process($element, $rendering_object);
  }

}
